==> 1.0.0-dev.1.tar.gz: lib/src/agent/action/action.dart <==
import 'package:flutter/services.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/root_action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/null_actions/null_action_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/web_request_timing_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

import '../../../tingyun_flutter_plugin.dart';
import 'counter.dart';

class TingyunRootActionImpl extends TingyunActionImpl
    implements TingyunRootAction {
  TingyunRootActionImpl.private(int key, MethodChannel methodChannel)
      : super.private(key, methodChannel);

  @override
  TingyunAction enterAction(String actionName) {
    Logger().d("enterAction actionName:${actionName}");
    if (_closed) {
      Logger().d("Action was closed already! Child action will have no effect.",
          logType: LogType.Warning);
      return TingyunNullAction();
    }

    if (StringUtils.isStringNullOrEmpty(actionName)) {
      Logger().d(
          "Action name was empty or null! Child action will have no effect.",
          logType: LogType.Warning);
      return TingyunNullAction();
    }

    int newKey = ActionCounter().getNewActionId();

    _methodChannel.invokeMethod("enterAction", <String, dynamic>{
      "name": actionName,
      "key": newKey,
      "enterTime": DateTime.now().millisecondsSinceEpoch.toString(),
      "parent": _key
    });
    return TingyunActionImpl.private(newKey, _methodChannel);
  }


  @override
  Future<TYWebRequestTiming> createWebRequestTiming(String url) async {
    String tag = await getRequestTag(url);
    return TingyunWebRequestTimingImpl(tag, url, _methodChannel);
  }

  @override
  bool isClosed() {
    return _closed;
  }
}

class TingyunActionImpl implements TingyunAction{
  final int _key;
  final MethodChannel _methodChannel;
  bool _closed = false;

  TingyunActionImpl.private(this._key, this._methodChannel);

  // @override
  // void reportError(String errorName, int errorCode) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Error can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(errorName) && errorCode != null) {
  //     _methodChannel.invokeMethod("reportErrorInAction", <String, dynamic>{
  //       "key": _key,
  //       "errorName": errorName,
  //       "errorCode": errorCode,
  //     });
  //   } else {
  //     Logger().d(
  //         "Error was not reported! ErrorName:$errorName errorCode:$errorCode",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportEvent(String eventName) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Event can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //   if (!StringUtils.isStringNullOrEmpty(eventName)) {
  //     _methodChannel.invokeMethod("reportEventInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": eventName,
  //     });
  //   } else {
  //     Logger().d(
  //         "Event was not reported because of missing name! EventName:$eventName",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportStringValue(String valueName, String value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! String can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName)) {
  //     _methodChannel
  //         .invokeMethod("reportStringValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "String was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportIntValue(String valueName, int value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Integer can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
  //     _methodChannel.invokeMethod("reportIntValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "Integer was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportDoubleValue(String valueName, double value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Double can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
  //     _methodChannel
  //         .invokeMethod("reportDoubleValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "Double was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }

  @override
  void leaveAction({Map<String, dynamic>props, int type = 0, String tag = ""}) {
    _closed = true;
    _methodChannel.invokeMethod("leaveAction", <String, dynamic>{
      "key": _key,
      "props": props,
      "type": type,
      "tag": tag,
      "leaveTime": DateTime.now().millisecondsSinceEpoch.toString()
    });
  }

  @override
  Future<String> getRequestTag(String url) async {
    return await _methodChannel.invokeMethod(
        "getRequestTag", <String, dynamic>{"key": _key, "url": url});
  }

  @override
  String getRequestTagHeader() {
    return "X-Tingyun";
  }

  @override
  Future<TYWebRequestTiming> createWebRequestTiming(String url) async {
    String tag = await getRequestTag(url);
    return TingyunWebRequestTimingImpl(tag, url, _methodChannel);
  }
}

==> 2.1.0.tar.gz: lib/src/agent/action/action.dart <==
import 'package:flutter/services.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/interface/root_action.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/null_actions/null_action_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/action/web_request_timing_impl.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/logger.dart';
import 'package:tingyun_flutter_plugin/src/agent/util/string_utils.dart';

import '../../../tingyun_flutter_plugin.dart';
import 'counter.dart';

class TingyunRootActionImpl extends TingyunActionImpl
    implements TingyunRootAction {
  TingyunRootActionImpl.private(int key, MethodChannel methodChannel)
      : super.private(key, methodChannel);

  @override
  TingyunAction enterAction(String actionName) {
    Logger().d("enterAction actionName:${actionName}");
    if (_closed) {
      Logger().d("Action was closed already! Child action will have no effect.",
          logType: LogType.Warning);
      return TingyunNullAction();
    }

    if (StringUtils.isStringNullOrEmpty(actionName)) {
      Logger().d(
          "Action name was empty or null! Child action will have no effect.",
          logType: LogType.Warning);
      return TingyunNullAction();
    }

    int newKey = ActionCounter().getNewActionId();

    _methodChannel.invokeMethod("enterAction", <String, dynamic>{
      "name": actionName,
      "key": newKey,
      "enterTime": DateTime.now().millisecondsSinceEpoch.toString(),
      "parent": _key
    });
    return TingyunActionImpl.private(newKey, _methodChannel);
  }


  @override
  Future<TYWebRequestTiming> createWebRequestTiming(String url) async {
    String tag = await getRequestTag(url);
    return TingyunWebRequestTimingImpl(tag, url, _methodChannel);
  }

  @override
  bool isClosed() {
    return _closed;
  }
}

class TingyunActionImpl implements TingyunAction{
  final int _key;
  final MethodChannel _methodChannel;
  bool _closed = false;

  TingyunActionImpl.private(this._key, this._methodChannel);

  // @override
  // void reportError(String errorName, int errorCode) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Error can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(errorName) && errorCode != null) {
  //     _methodChannel.invokeMethod("reportErrorInAction", <String, dynamic>{
  //       "key": _key,
  //       "errorName": errorName,
  //       "errorCode": errorCode,
  //     });
  //   } else {
  //     Logger().d(
  //         "Error was not reported! ErrorName:$errorName errorCode:$errorCode",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportEvent(String eventName) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Event can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //   if (!StringUtils.isStringNullOrEmpty(eventName)) {
  //     _methodChannel.invokeMethod("reportEventInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": eventName,
  //     });
  //   } else {
  //     Logger().d(
  //         "Event was not reported because of missing name! EventName:$eventName",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportStringValue(String valueName, String value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! String can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName)) {
  //     _methodChannel
  //         .invokeMethod("reportStringValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "String was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportIntValue(String valueName, int value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Integer can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
  //     _methodChannel.invokeMethod("reportIntValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "Integer was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }
  //
  // @override
  // void reportDoubleValue(String valueName, double value) {
  //   if (_closed) {
  //     Logger().d(
  //         "Action was closed already! Double can not be reported on this action!",
  //         logType: LogType.Error);
  //     return;
  //   }
  //
  //   if (!StringUtils.isStringNullOrEmpty(valueName) && value != null) {
  //     _methodChannel
  //         .invokeMethod("reportDoubleValueInAction", <String, dynamic>{
  //       "key": _key,
  //       "name": valueName,
  //       "value": value,
  //     });
  //   } else {
  //     Logger().d(
  //         "Double was not reported because of missing values! ValueName:$valueName Value:$value",
  //         logType: LogType.Error);
  //   }
  // }

  @override
  void leaveAction({Map<String, dynamic>? props, int type = 0, String tag = ""}) {
    _closed = true;
    _methodChannel.invokeMethod("leaveAction", <String, dynamic>{
      "key": _key,
      "props": props,
      "type": type,
      "tag": tag,
      "leaveTime": DateTime.now().millisecondsSinceEpoch.toString()
    });
  }

  @override
  Future<String> getRequestTag(String url) async {
    return await _methodChannel.invokeMethod(
        "getRequestTag", <String, dynamic>{"key": _key, "url": url});
  }

  @override
  String getRequestTagHeader() {
    return "X-Tingyun";
  }

  @override
  Future<TYWebRequestTiming> createWebRequestTiming(String url) async {
    String tag = await getRequestTag(url);
    return TingyunWebRequestTimingImpl(tag, url, _methodChannel);
  }
}
